﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Ozeki.VoIP.SIP;

namespace MessageManipulatorExample
{
    /// <summary>
    /// Replaces the value of the "Organization" header field of the incoming and outgoing message to a custom value.
    /// </summary>
    class SIP_OrganizationManipulator : ISIPMessageManipulator
    {
        public string CustomInValue{ get; set; }
        public string CustomOutValue { get; set; }

        /// <summary>
        /// Replaces the value of the "Organization" header field of the incoming message to a custom value.
        /// </summary>
        public string PerformReceivedMessage(string incomingMessage)
        {
            if (string.IsNullOrEmpty(CustomInValue))
                return null;

            return ReplaceOrganizationValue(incomingMessage, true);
        }

        /// <summary>
        /// Replaces the value of the "Organization" header field of the incoming or outgoing message to a custom value.
        /// </summary>
        public string PerformSendMessage(string outGoingMessage)
        {
            if (string.IsNullOrEmpty(CustomOutValue))
                return null;

            return ReplaceOrganizationValue(outGoingMessage, false);
        }

        /// <summary>
        /// Replaces the value of the "Organization" header field of the incoming or outgoing message to a custom value.
        /// </summary>
        private string ReplaceOrganizationValue(string sipMessage, bool incomingMessage)
        {
            string[] headers = sipMessage.Split(new string[] { "\r\n" }, StringSplitOptions.None);
            bool found = false;
            int breakPosition = -1;
            for (int i = 0; i < headers.Length; i++)
            {
                if (string.IsNullOrEmpty(headers[i]))
                {
                    breakPosition = i;
                    break;
                }

                if (headers[i].StartsWith("Organization"))
                {
                    string value = (incomingMessage) ? CustomInValue : CustomOutValue;
                    headers[i] = "Organization: " + value;
                    breakPosition = i;
                    found = true;
                    break;
                }
            }

            if (incomingMessage && !found)
            {
                Array.Resize(ref headers, headers.Length + 1);
                if (breakPosition != -1)
                {
                    headers[breakPosition] = "Organization: " + CustomInValue;
                    headers[headers.Length - 1] = "";
                }
                else
                    headers[headers.Length - 1] = "Organization: " + CustomInValue;
            }

            string manipulatedSIPMessage = string.Join("\r\n", headers);
            return manipulatedSIPMessage;
        }

        /// <summary>
        /// Adds a "Organization" header field (if not exists) to the outgoing SIP message.
        /// </summary>
        public IEnumerable<PreparedExtensionHeader> PrepareAdditionalHeaders(string outGoingMessage)
        {
            if (string.IsNullOrEmpty(CustomOutValue))
                return null;

            // see if headers list contains Organization
            string[] sipHeaders = outGoingMessage.Split(new string[] { "\r\n" }, StringSplitOptions.None);
            foreach (string header in sipHeaders)
            {
                if (header.StartsWith("Organization"))
                    return null;
            }
            
            List<PreparedExtensionHeader> list = new List<PreparedExtensionHeader>();
            list.Add(new PreparedExtensionHeader("Organization", CustomOutValue));
            return list;
        }
    }
}
